@@ -30,6 +30,9 @@ import requests
 # are processed.
 #
 # This is supported in Python 3.7+
+from .exceptions import AnamlError
+from .models.summary_statistics import SummaryStatistics
+
 if typing.TYPE_CHECKING:
     import pandas
     import pyspark.sql
@@ -65,8 +68,6 @@ from . import version
 
 __version__ = version.__version__
 
-log = logging.getLogger(__name__)
-
 _NO_FEATURES = """No feature instances were generated for the following """
 _NO_FEATURES += """features:\n{the_features}.\n"""
 _NO_FEATURES += """This could be because the underlying dataset was empty, """
@@ -80,7 +81,14 @@ class Anaml:
     _bigquery_client_instance: Optional[bigquery.Client] = None
     _s3fs_client_instance: Optional[s3fs.S3FileSystem] = None
 
-    def __init__(self, url: str, apikey: str, secret: str, ref: Optional[Ref] = None):
+    def __init__(
+        self,
+        url: str,
+        apikey: str,
+        secret: str,
+        ref: Optional[Ref] = None,
+        log: Optional[logging.Logger] = None
+    ):
         """Create a new `Anaml` instance.
 
         Access to the API requires a Personal Access Token which can be obtain on the users profile page
@@ -91,10 +99,12 @@ class Anaml:
             apikey: API key for Personal Access Token.
             secret: API secret for Personal Access Token.
             ref: The BranchRef or CommitRef reference to act on.
+            log: Optional logger to use. If omitted, a logger will be created.
         """
         self._url = url
         self._token = base64.b64encode(bytes(apikey + ':' + secret, 'utf-8')).decode('utf-8')
         self._headers = {'Authorization': 'Basic ' + self._token}
+        self._log = log or logging.getLogger('anaml_client.Anaml')
         if ref is not None:
             self._ref = {ref.adt_type: ref.ref}
         else:
@@ -305,20 +315,21 @@ class Anaml:
         _ = self._int_or_handle_errors(r)
         return self.get_feature_by_id(int(feature.id.value))
 
-    def preview_feature(self, feature: Feature) -> List:
+    def preview_feature(self, feature: Union[List[Feature], Feature]) -> List[SummaryStatistics]:
         """Returns list of feature statistics.
 
         Args:
-            feature: a Feature object
+            feature: a Feature object or list of feature objects.
         """
-        r = self._post("/feature-preview", json={"features": [feature.to_json()]})
+        if isinstance(feature, list):
+            req = {"features": [f.to_json() for f in feature]}
+        else:
+            req = {"features": [feature.to_json()]}
+        r = self._post("/feature-preview", json=req)
         result = self._json_or_handle_errors(r)
-
         return [
-            fs
-            for pd in self._to_list(result.get("previewData"))
-            for fss in self._to_list(pd.get("featureStatistics"))
-            for fs in fss
+            SummaryStatistics.from_json(fs)
+            for fs in result.get('previewData', {}).get('featureStatistics', [])
         ]
 
     def plot_feature_preview(self, feature: Feature) -> None:
@@ -778,10 +789,14 @@ class Anaml:
               Otherwise, a Pandas data frame will be created and returned.
         """
         if run.status != RunStatus.Completed:
-            log.debug(f"Attempted to load data from feature store run id={run.id.value}, status={run.status.value}")
+            self._log.debug(
+                f"Attempted to load data from feature store run id={run.id.value}, status={run.status.value}"
+            )
             raise ValueError("The feature store run is not complete")
-        # TODO: We should thing about using Version here.
+
+        # TODO: We should think about using Version here.
         store = self.get_feature_store_by_id(run.featureStoreId.value)
+
         # Loop through the destinations and attempt to load them. They should all contain the same data, so we'll
         # take the first one we find that actually returns a dataframe.
         dataframe = None
@@ -813,7 +828,7 @@ class Anaml:
                 # TODO: Implement support for loading data from Hive.
                 # TODO: Implement support for loading data from HDBC.
                 else:
-                    log.debug(f"Cannot load table data from {type(dest).__name__}; skipping.")
+                    self._log.debug(f"Cannot load table data from {type(dest).__name__}; skipping.")
             elif isinstance(dest_ref, FolderDestinationReference):
                 if isinstance(dest, GCSDestination) and spark_session is not None:
                     url = "gs://{bucket}/{prefix}/".format(
@@ -887,9 +902,9 @@ class Anaml:
                         format=dest.fileFormat
                     )
                 else:
-                    log.debug(f"Cannot load folder data from {type(dest).__name__}; skipping.")
+                    self._log.debug(f"Cannot load folder data from {type(dest).__name__}; skipping.")
             else:
-                log.debug(f"Cannot load data from {type(dest_ref).__name__} references; skipping.")
+                self._log.debug(f"Cannot load data from {type(dest_ref).__name__} references; skipping.")
             if dataframe is not None:
                 return dataframe
 
@@ -954,42 +969,40 @@ class Anaml:
     def _to_list(gotten: Optional[A]) -> List[A]:
         return [] if gotten is None else [gotten]
 
-    @staticmethod
-    def _warn_empty_stats(features: List[str]):
+    def _warn_empty_stats(self, features: List[str]):
         if features:
-            log.warning(_NO_FEATURES.format(thefeatures=', '.join(features)))
+            self._log.warning(_NO_FEATURES.format(thefeatures=', '.join(features)))
 
-    @staticmethod
-    def _json_or_handle_errors(r):
+    def _json_or_handle_errors(self, r: requests.Response):
         if r.ok:
             try:
                 result = r.json()
                 return result
             except json.JSONDecodeError:
                 # Sorry, (no or invalid) JSON here
-                log.error("No or invalid JSON received from server")
-                log.error("Response content: " + r.content)
+                self._log.error("No or invalid JSON received from server")
+                self._log.error("Response content: " + r.text)
                 r.raise_for_status()
         else:
-            if "errors" in r:
-                log.error(json.dumps(r._get("errors"), indent=4))
-
+            response_json = r.json()
+            if "errors" in response_json:
+                ex = AnamlError.from_json(response_json)
+                self._log.error("The Anaml server reported an error", ex)
+                raise ex
         r.raise_for_status()
 
-    @staticmethod
-    def _int_or_handle_errors(r):
+    def _int_or_handle_errors(self, r):
         if r.ok:
             return int(r.text)
         else:
             if "errors" in r:
-                log.error(json.dumps(r._get("errors"), indent=4))
-        log.error(f"HERE: {r.text}")
+                self._log.error(json.dumps(r._get("errors"), indent=4))
+        self._log.error(f"HERE: {r.text}")
         r.raise_for_status()
 
-    @staticmethod
-    def _warn_empty_feature_stats(features: List[str]):
+    def _warn_empty_feature_stats(self, features: List[str]):
         if features:
-            log.warning(_NO_FEATURES.format(thefeatures=', '.join(features)))
+            self._log.warning(_NO_FEATURES.format(thefeatures=', '.join(features)))
 
     @staticmethod
     def _file_format_suffix(fmt: FileFormat):
